#
#
tmpdir=$Proot/install/tmp
if [ -s $tmpdir/bbb ] ; then
    waitMsgPid=`cat $tmpdir/bbb`
    kill $waitMsgPid 2>/dev/null
    \rm -rf $tmpdir/bbb
fi
$bin_dir/msgDialog "$@"
msgStatus=$?
$BOURNE $bin_dir/waitMsgf
exit $msgStatus
