#
#
tmpdir=$Proot/install/tmp
if [ -s $tmpdir/bbb ] ; then
    waitMsgPid=`cat $tmpdir/bbb`
    kill $waitMsgPid 
    \rm -rf $tmpdir/bbb
fi
$bin_dir/toplevel "$@"  
topStatus=$?
case $1 in
    -1)        break;;
     11|12|13) if [ $topStatus -gt 0 ]; then
                   $BOURNE $bin_dir/waitMsgf  
               else
                   break;   # form status is test, config, sysconfig and userconfig
               fi
               ;;
     0)
	 if [ $topStatus -eq 250 ]; then
                  # exit status is Quit from Main menu
	    break;
         else
            $BOURNE $bin_dir/waitMsgf 
	 fi
	 ;;
     *)    $BOURNE $bin_dir/waitMsgf 
esac
exit $topStatus
